/* homenode-project
   Autor: Roland Praml
   
   hardware spezifische Funktionen
   
   benutzt die Register r3 - r6 !!! 
*/

#include <avr/io.h>
#include <avr/interrupt.h>
#include <avr/signal.h>
#include "main.h"
#include "hardware.h"
#include "protocol.h"


// Int1 als TX Control verwenden (fr Konvertermodul), ansonsten als Phasenanschnitt
// #define INT1_TXCTL

unsigned char register noisecanceler  asm("r4");
unsigned char register tmp_save_sreg  asm("r5");
unsigned char register timer asm("r6"); // fr Phasenanschitt (zhlt von 255 bis ca 100 pro Nulldurchgang)
unsigned char cnt;	// Durchlaufender Zhler
unsigned char agc;  // PWM-Ausgang fr Gain


// Interrupt Routinen fr Osc. Steuerung , ca 12  byte
// Wenn ein Interrupt auftritt, dann wird der Trgeroszillator freigegeben
SIGNAL(SIG_INTERRUPT0) __attribute__ ((naked));
SIGNAL(SIG_INTERRUPT0) { 
		asm("	in r5, __SREG__");
		TXLO;
		if (bit_is_clear(PIND, PD2)) { // Sender muss eingeschaltet werden
			TXHI;
		} 
		asm("	out __SREG__, r5	\n" \
			" 	reti");
}


#ifdef INT1_TXCTL
SIGNAL(SIG_INTERRUPT1) __attribute__ ((naked));
SIGNAL(SIG_INTERRUPT1) { 
		asm("	in r5, __SREG__");
		
		TXLO;
		if (bit_is_clear(PIND, PD3)) { // Sender muss eingeschaltet werden
			TXHI;
		} 
		
		asm("	out __SREG__, r5	\n" \
			" 	reti");
}

#else
SIGNAL(SIG_INTERRUPT1) __attribute__ ((naked));
SIGNAL(SIG_INTERRUPT1) { // 30 Byte  
		asm("	in r5, __SREG__");

		asm(" eor r6, r6 	\n"\
			" dec r6 \n"\
			"	out __SREG__, r5	\n" \
			" reti"); // Timer bei 0-durchgang zurcksetzen
}
#endif




/* Timerinterrupt, wird alle 4 MHz/256 = 15,625 bzw 64 S aufgerufen

*/

#define NOISE_QUAL 128
// Triggerschwelle
#define NOISE_VAL 16

SIGNAL(SIG_OVERFLOW0) { // 178 Byte 
	
	unsigned char tmp;
	unsigned char i;

	// Digitaler Filter zum Unterdrcken von Spikes
	noisecanceler = noisecanceler >> 1; // dividieren durch 2
	// asm("lsr r4"); 
	if (bit_is_clear(ACSR, ACO)) {
		noisecanceler |= NOISE_QUAL; // LSB setzen
	}
		
	if (noisecanceler > (NOISE_QUAL*2 - 1) - NOISE_VAL) {
			RXHI;
	} 
	if (noisecanceler < NOISE_VAL) {
			RXLO;
	}


	if (cnt > agc) PORTB &= ~(1 << PB0); else PORTB |= (1 << PB0); // PWM-Referenzspannung erzeugen
	
	// Phasenanschnittsteuerung
	tmp = 0;
	if (timer > 128 ) { // wir dimmen nur, wenn der Zhler noch grer 128 ist
		// die Bits 0..3 in tmp setzen wenn cfg[0..3] >= timer
		i=4;
		do {
			i--;
			tmp = tmp << 1;
			if (timer <= cfg[i]) tmp |= 1; // Output setzen
		} while(i);
		
	}
	if (timer > 1) timer--;

	/*tmp2=0;
	if (cfg[0] > 128) 	tmp |= 1; // Output setzen
	if (cfg[1] > 128) 	tmp |= 2; // Output setzen
	if (cfg[2] > 128) 	tmp |= 4; // Output setzen
	if (cfg[3] > 128) 	tmp |= 8; // Output setzen
	*/

	/*
	// direkt geschaltete Ports	
	for (i = 0; i < 4; i++) {
		if ( !(setup & (1 << i)) ) { //  Dieser Port ist kein Dimm-Port
			if (cfg[i] > 128) {	
				tmp |= (1 << i); // Output setzen
			} else {
				tmp &= ~(1 << i); // Output lschen

          	}
		}
	 }*/
	 
	PORTC = (tmp ^ outxor ) & 0x0F;

	
	// Steuerung fr queuetimer
	cnt++;

	if (cnt == 0) {
		waitqueue=0; // alle 16 ms queue freigeben
		send_fifo();
	}

	if ((cnt & 0x07) == 0) {
//		sei();
	 	keyhandler((cnt >> 3) & 0x03);
	}
}


void inline init_hardware(void) {
	unsigned int i;
	unsigned char r;

	agc=10;								// Gain fr Empfnger

	TXLO;		 						// Output 140 KHz Osc.
#ifdef ATMEGA8
	TCCR1A = 1 << COM1A0;				// Toggle OC1 Line on Compare
	TCCR1B = 1 << WGM12 | 1 << CS10 ;	// Reset on compare, luft mit 4 MHz
	OCR1A=8;							// bis 16 zhlen  (4MHz / 32 = 125 kHz)
#else
	TCCR1A = 1 << COM10;				// Toggle OC1 Line on Compare
	TCCR1B = 1 << CTC1 | 1 << CS10 ;	// Reset on compare, luft mit 4 MHz
	OCR1=8;							// bis 16 zhlen  (4MHz / 32 = 125 kHz)
#endif

    
	// Timer fr Phasenanschnitt, 64S, Helligkeit kann in ca 156 Schritten geregelt werden
	// Timer zhlt mit 4 MHZ, SIGNAL-Interrupt alle 4MHz/256 = 15,625 KHz
	// TCCR0 = 1 << CS02 | 1 << CS00;
    
	TCCR0 =  1 << CS00;	// Clock
	TIMSK = 1 << TOIE0;	// Interruot aktivieren

	// Steuerport PD2 und evtl. PD3 lassen den Osc. schwingen, dadurch kann
	// man einfach die interne UART verwenden

	PORTD = 0 << PD0 	| // Pullup RXpin
			0 << PD1	| // UART-TX
			0 << PD2 	| // Pullup TX-Intpin
			0 << PD3	| // Phasenanschnitt
			0 << PD4    | // Ausgang vom Komparator-> geht in RX
			0 << PD5	| // reserve
			0 << PD6	| // + Komp
			0 << PD7	| // - Komp
			0;
	DDRD  = 0 << PD2 	| // Pullup TXpin
			0 << PD3	| // Phasenanschnitt Interrupt
			1 << PD4	| // Ausgang Komp
			0 << PD6	| // + Komp
			0 << PD7	| // - Komp
			0;

	PORTB = 1 << PB0	|	// Referenz-PWM fr AGC
			0 << PB1	|	// Modulierter Ausgang
			0 << PB2	|	// 
			1 << PB3	|	// Taster 1 Pullup
			1 << PB4	|	// Taster 2 Pullup
			1 << PB5	|	// Taster 3 Pullup
			0;
	DDRB  = 1 << PB0	|	// Referenz
			0 << PB1	|	// Modulierter Ausgang
			0 << PB2	|	// 
			0 << PB3	|	// Taster 1 Pullup
			0 << PB4	|	// Taster 2 Pullup
			0 << PB5	|	// Taster 3 Pullup
			0;
	
	PORTC = 1 << PC0	|	// Triac 1
			1 << PC1	| 	// Triac 2
			1 << PC2	| 	// Triac 3
			0;
			 
	DDRC  = 1 << PC0	|	// Triac 1
			1 << PC1	| 	// Triac 2
			1 << PC2	| 	// Triac 3
			0;

	
	// Interrupts fr TX-Osc und Phasenanschnitt
	MCUCR |= (1 << ISC00 ) | (1 << ISC10 ); 	// Toggle lst Interrupt aus
	GIMSK |= (1 << INT0)   | (1 << INT1);		// beide Interrupts einschalten
	
	TXLO;	// nicht senden										
	RXLO;	// nichts zu empfangen
	
	// Steigt die Spannung an AIN1 ber die Referenzsspannung welche per PWM erzeugt wird,
	// wird ein Interrupt ausgelst, welcher PD4 steuert, welcher wiederrum zum RX -Pin der UART geht

/*	ACSR = 	0 << ACD	| 	// Comparator enable
		   	0 << AINBG	| 	// keine Referenz
			0 << ACIE	|	// Interrupt enable
			0 << ACIS0	|
			0 << ACIS1  |	// Trigger on toggle
			0;
*/

	// UART
	  
  	UCSRB = (1 << TXEN) | (1 << RXEN);  // RX und TX aktivieren
  	UBRRH = 0;
#ifdef ATMEGA8
	UBRRL = 207;	// Baudrate 1200 bei 4MHz
#else  	
	UBRR = 207;	    // Baudrate 1200 bei 4MHz
#endif
	// LED-Setup    
	LED1OFF
	LED2OFF
	LED3OFF
	LED1ON

	for (r=1; r < 4; r++) {
		for (i = 0; i < 65000; i++) asm("nop");
		LED3ON
		for (i = 0; i < 65000; i++) asm("nop");
		LED3OFF
	}


	setup = 1 * DIM1	|
			1 * DIM2	|
			0 * DIM3	|
			0 * DIM4	|
			1 * KEY1	|
			1 * KEY2	|
			1 * KEY3	|
			1 * SETUP_ZERO_CROSS	|

						0;

 	outxor=	1 * DIM1	|	// Xor Mask da Triacs invertiert angesteuert werden
			1 * DIM2	|
			1 * DIM3	|
			0;

}
