/* homenode-project
   Autor: Roland Praml
   
   hardware spezifische Funktionen
   
   
*/

#include <avr/io.h>
#include <avr/interrupt.h>
#include <avr/signal.h>
#include "main.h"
#include "hardware.h"
#include "protocol.h"
#include "register.h"






unsigned char volatile ZeroCrossFlag; 


// Interrupt Routinen fr Osc. Steuerung , ca 12  byte
// Wenn ein Interrupt auftritt, dann wird der Trgeroszillator freigegeben
INTERRUPT(SIG_INTERRUPT0) __attribute__ ((naked));
INTERRUPT(SIG_INTERRUPT0) { 
	asm("	in r5, __SREG__");
	TXLO;
	if (bit_is_clear(PIND, PD2)) { // Sender muss eingeschaltet werden
		TXHI;
	} 
	asm("	out __SREG__, r5	\n" \
		" 	reti");
}


#ifdef INT1_TXCTL
// Interrupt fr zweiten TX
INTERRUPT(SIG_INTERRUPT1) __attribute__ ((naked));
INTERRUPT(SIG_INTERRUPT1) { 
	asm("	in r5, __SREG__");
	TXLO;
	if (bit_is_clear(PIND, PD3)) { // Sender muss eingeschaltet werden
		TXHI;
	} 
		
	asm("	out __SREG__, r5	\n" \
		" 	reti");
}

#else
// Interrupt fr Nulldurchgang: 
INTERRUPT(SIG_INTERRUPT1) { // 30 Byte  
	ZeroCrossFlag |= 1;	
}
#endif



/* Timerinterrupt, wird alle 4 MHz/256 = 15,625 bzw 64 S aufgerufen

*/







#ifdef NEG_HW
#define SETCHAN(CH) cbi(POWER_PORT,CH)
#define CLRCHAN(CH) sbi(POWER_PORT,CH)
#else
#define SETCHAN(CH) sbi(POWER_PORT,CH)
#define CLRCHAN(CH) cbi(POWER_PORT,CH)
#endif




SIGNAL(SIG_OVERFLOW0) { // 178 Byte , 64 S

	// 17 Cycles fr Prambel	
	unsigned char register tmp;
	unsigned char register cnt;
	static unsigned char clockcnt;	// Durchlaufender Zhler


	tmp = ADCH;
	noisecanceler *= 2;
	if (tmp > threshold) {
		noisecanceler++; // eine 1 rein schieben
	}


/* 
    2   4   6   8
+   1   3   5   7
=  1+2 3+4 5+6 7+8

	1+2    3+4 
+   5+6    7+8
=  12+56   34+78

   1256+3478
*/
 
	cnt=noisecanceler;   // gesetzte bits zhlen                 
 	cnt= ((cnt>>1)&0x55)+(cnt&0x55);
 	cnt= ((cnt>>2)&0x33)+(cnt&0x33);
 	cnt= ((cnt>>4)+cnt)&0xf;

	if (cnt >= 6 ) {		// mind 5 Bits gesetzt
			RXLO;  // eine "invertierte" 1 senden
	} 
	
	if (cnt < 3) {			// mind 5 Bits 0
		 	RXHI;
	}   

	/*if (bytereceived) {
		bytereceived--;
	} else {
		noisesignal = noisesignal - (noisesignal >> 8);
		noisesignal += tmp;
	//	threshold = (noisesignal >> 8) *4;

	}*/


// Triac-Ansteuerung	
	if (timer > 127) {
		// hier passiert die eigentliche Action
		// Ausgnge setzen, wenn der Timer "vorbei" kommt
		if (timer == cfg[0]) SETCHAN(0);
		if (timer == cfg[1]) SETCHAN(1); 
		if (timer == cfg[2]) SETCHAN(2);
		if (timer == cfg[3]) SETCHAN(3);
		timer--;
		ZeroCrossFlag = CFG_PWM; // es kann hier keinenNulldurchgang geben, wurde einer registriert, so ist dies falsch!
	} else 	{ 
		
		if (ZeroCrossFlag > 1) { // PWM-Mode
			timer--;
			if (timer < grundoffset) { // Grundhelligkeit
				timer=255;
				if (cfg[0] < 255) CLRCHAN(0);
				if (cfg[1] < 255) CLRCHAN(1);
				if (cfg[2] < 255) CLRCHAN(2);
				if (cfg[3] < 255) CLRCHAN(3);
			}
		
		} else { //Phasenanschnitt
			// kurz vor dem Nulldurchgang Kanal wieder abschalten
		
			// wir sind kurz vor dem Nullduchgang.
			if (ZeroCrossFlag) {
				timer=255;
				ZeroCrossFlag=0;
			}
			
			if (cfg[0] < 255) CLRCHAN(0);
			if (cfg[1] < 255) CLRCHAN(1);
			if (cfg[2] < 255) CLRCHAN(2);
			if (cfg[3] < 255) CLRCHAN(3);

		}	

	} 

	clockcnt++;

	if (clockcnt >= 16) {
		wait_1ms=0; // alle 1 ms queue freigeben
		clockcnt=0;
	}

}



void inline init_hardware(void) {



	TXLO;		 						// Output 140 KHz Osc.
	TCCR1A = 1 << COM1A0;				// Toggle OC1 Line on Compare
	TCCR1B = 1 << WGM12 | 1 << CS10 ;	// Reset on compare, luft mit 4 MHz
	OCR1A=CFG_TXHF;						// bis 16 zhlen  (4MHz / 32 = 125 kHz)

    
	// Timer fr Phasenanschnitt, 64S, Helligkeit kann in ca 156 bzw 128 effektiven
	// Schritten geregelt werden
	// Timer zhlt mit 4 MHZ, SIGNAL-Interrupt alle 4MHz/256 = 15,625 KHz

  	TCCR0 =  1 << CS00;	// Timer0 Prescaler: Clock
	TIMSK = 1 << TOIE0;	// Interruot aktivieren

	// Steuerport PD2 und evtl. PD3 lassen den Osc. schwingen, dadurch kann
	// man einfach die interne UART verwenden

	PORTD = 0 << PD0 	| // Pullup RXpin
			0 << PD1	| // UART-TX
			0 << PD2 	| // Pullup TX-Intpin
			0 << PD3	| // Phasenanschnitt
			0 << PD4    | // Ausgang vom Komparator-> geht in RX
			0 << PD5	| // reserve
			0 << PD6	| // + Komp
			0 << PD7	| // - Komp
			0;
	DDRD  = 0 << PD2 	| // Pullup TXpin
			0 << PD3	| // Phasenanschnitt Interrupt
			1 << PD4	| // Ausgang Komp
			0 << PD6	| // + Komp
			0 << PD7	| // - Komp
			0;

	// Real: PB, Debug PD
	KEY_PORT = (0 << PB0	|	// Referenz-PWM fr AGC
			0 << PB1	|	// Modulierter Ausgang
			1 << PB2	|	// 
			1 << PB3	|	// Taster 1 Pullup
			1 << PB4	|	// Taster 2 Pullup
			1 << PB5	|	// Taster 3 Pullup
			0 )^ (CFG_KEYXOR << 2); // Pullups fr 230V-Eingnge weg
	KEY_DDR  = 0 << PB0	|	// Referenz
			0 << PB1	|	// Modulierter Ausgang
			0 << PB2	|	// 
			0 << PB3	|	// Taster 1 Pullup
			0 << PB4	|	// Taster 2 Pullup
			0 << PB5	|	// Taster 3 Pullup
			0;
	// Real: PC, Debug PB
#ifdef NEG_HW			 

	POWER_PORT = 1 << PC0	|	// Triac 1
			1 << PC1	| 	// Triac 2
			1 << PC2	| 	// Triac 3
			1 << PC3	| 	// Triac 3
			0;
#else
	POWER_PORT = 0 << PC0	|	// Triac 1
			0 << PC1	| 	// Triac 2
			0 << PC2	| 	// Triac 3
			0 << PC3	| 	// Triac 3
			0;
#endif

	POWER_DDR  = 1 << PC0	|	// Triac 1
			1 << PC1	| 	// Triac 2
			1 << PC2	| 	// Triac 3
			1 << PC3	| 	// Triac 3
			0;

	
	// Interrupts fr TX-Osc und Phasenanschnitt
	MCUCR |= (1 << ISC00 ) | (1 << ISC10 ); 	// Toggle lst Interrupt aus
	GIMSK |= (1 << INT0)   | (1 << INT1);		// beide Interrupts einschalten
	
	TXLO;	// nicht senden										
	RXLO;	// nichts zu empfangen
	
	// Steigt die Spannung an AIN1 ber die Referenzsspannung welche per PWM erzeugt wird,
	// wird ein Interrupt ausgelst, welcher PD4 steuert, welcher wiederrum zum RX -Pin der UART geht


	ADMUX =  1 << REFS0 |
			 1 << REFS1 |	// 2,56 V
		     1 << ADLAR | 5 ; // Laeft adjust und Port 5 selektieren

	ADCSRA = 1 << ADEN | 1 << ADSC | 1 << 5 | 4; // Free running, 250 KHz converison frequency

 			  
  	UCSRB = (1 << TXEN) | (1 << RXEN);  // RX und TX aktivieren
  	UBRRH = 0;

	UBRRL = 207;	// Baudrate 1200 bei 4MHz
//	UBRRL = 230;	// Baudrate 1200 bei 4,43 MHz




	cfg[0] = CFG_CHAN0;
	cfg[1] = CFG_CHAN1;
	cfg[2] = CFG_CHAN2;
	cfg[3] = CFG_CHAN3;

	ZeroCrossFlag	=	CFG_PWM;
	setup 	 		=	CFG_SETUP;
	fallback 		=	CFG_FALLBACK;
	grundoffset		=	CFG_OFFSET;
	tastdown		=   0;
    threshold		=	CFG_THRESHOLD; // 150 mV

}






