/*************************************************************************
Homenode, by Roland Praml (pram at gmx dot de)


             RESET   V	PC5 (ADC5)
   +-----(RXD) PD0  	PC4 (ADC4)
   |  +--(TXD) PD1 		PC3 (ADC3)
   |  +-(INT0) PD2 		PC2 (ADC2)	TRIAC3
 ~-(----(INT1) PD3		PC1 (ADC1)	TRIAC2
   +------(T0) PD4		PC0 (ADC0)	TRIAC1
               VCC-+ +--AGND
               GND-(-+	AREF
             XTAL1 +----AVCC
             XTAL2		PB5 (SCK)   LED1 / Taster 1
          (T1) PD5		PB4 (MISO)  LED2 / Taster 2
        (AIN0) PD6		PB3 (MOSI)  LED3 / Taster 3
+-------(AIN1) PD7		PB2 (SS) 
+-[100k]-(ICP) PB0		PB1 (OC1)  
|
|   22F
+---||----+
+--[10k]--+---GND

             
			 RESET   V	PC5 (ADC5)
         (RXD) PD0  	PC4 (ADC4)
 	  +--(TXD) PD1 		PC3 (ADC3)
      +-(INT0) PD2 		PC2 (ADC2)
        (INT1) PD3		PC1 (ADC1)
Trffner (T0) PD4		PC0 (ADC0)
               VCC		AGND
               GND		AREF
             XTAL1		AVCC
             XTAL2		PB5 (SCK)   LED1
Klingel   (T1) PD5		PB4 (MISO)  LED2
Otreppe (AIN0) PD6		PB3 (MOSI)  LED3
ITreppe (AIN1) PD7		PB2 (SS)
         (ICP) PB0		PB1 (OC1)

**************************************************************************/

#include <avr/io.h>
#include <avr/interrupt.h>
#include <avr/pgmspace.h>
#include <avr/signal.h>
#include <avr/eeprom.h>
#include "main.h"
#include "protocol.h"
#include "hardware.h"
#include "keyhandler.h"
#include "register.h"

/**********************************************************************************
 * Diese Funktion bernimmt die "Dimto-Steuerung". d.H der AVR kann selbststndig
 * einen Kanal zu einem Wert "hindimmen" oder einen Kanal eine bestimmte Zeit lang
 * einschalten. Der Wert zu dem gedimmt werden soll, liegt in dim_to[i].
 * dim_to[i] = 0	: nichts wird gemacht
 * dim_to[i] < 127	: es wird dim_to[i] * dim_time[i] mS der Kanal eingeschaltet
 * dim_to[i] >= 127	: es wird alle dim_time[i] mS der Kanal um einen Schritt 
 *                    in Richtung dim_to[i] angepasst, bis dieser erreicht ist
 *					  es wird dann ein EVENT_DIMFINISH ausgelst
 **********************************************************************************/


void inline dimhandler() {
	unsigned char i;
	for ( i = 0; i < 4; i++ ) { // 4 Channels

		if ( dim_to[i] ) { // berhaupt ein Befehl da?

			if ( dim_to[i] >= 127 ) { // Modus: hindimmen
				dim_cnt[i]++;

				if (dim_cnt[i] >= dim_time[i]) { // lang genug gewartet
					dim_cnt[i]=0;

					if (dim_to[i] < cfg[i]) { // in welche Richtung?
						cfg[i]--;
					} else if (dim_to[i] > cfg[i]){
						cfg[i]++;
					} else {
						dim_to[i]=0; // fertig, evtl event auslsen
						send_event( EVENT_DIMFINISH | i);
					}
				}
	
				
			} else { // wir sind im "Impuls-modus" zum kurzen Ansteuern vom Gangrelais z.B
				dim_cnt[i]++;

				if (dim_cnt[i] >= dim_time[i]) { // lang genug gewartet
					dim_cnt[i]=0;

					dim_to[i]--;
					if (dim_to[i] > 0) {
						cfg[i]=255;
					} else {
						cfg[i]=0;
					}
				}

			}
		}
	}
}

/***********************************************************************************
 * Das Hauptprogramm initialisiert die Hardware und verschickt regelmig die Messages
 * der Queue, ausserdem frag es die Tasten ab
 ***********************************************************************************/

int main(void) { // 96 byte

	unsigned char ms_cnt=0;
	init_hardware(); // AVR Register und Interrupts initialisieren
	send_event( EVENT_POWERON );

	sei();
/*	while(1) {
		uart_send_byte(0x55);
	}*/
	
	for(;;) { // Hauptprogramm

		while(wait_1ms) { // Diese Schleife wird alle 1 ms verlassen

			if (UCSRA & (1 << RXC))	{ // liegt ein Bit an der seriellen an?
						
						uart_receive_byte(inp(UDR));
			}

			keyhandler();
			

			// hier weitere Programmteile einbauen. Kein Programmteil darf lnger als 8mS busy sein,
			// da sonst die Bytes nicht korrekt empfangen werden knnen
		};
		wait_1ms=1; // auf die nchste ms warten
		ms_cnt++;
		if (ms_cnt >= KEYUPDPERIOD) {
			ms_cnt=0;
			updatekeydelays();
			sendqueue(); // Hier werden eventuelle Pakete noch aus der Queue verschickt, dies dauert ca. 80 ms
		}

		
		dimhandler();

	}
	return 0;

}

