/*************************************************************************
Title:    SMS Alarm-notifier
Author:   Roland Praml  <praml.roland@t-online.de>  
Date:     March 2003
Software: AVR-GCC with GNU binutiils
          LCD Routines from Peter Fleury
Hardware: HD44780 2x16 compatible LCD text display
          ATS90S8515



             RESET   V	PC5 (ADC5)
   +-----(RXD) PD0  	PC4 (ADC4)
   |  +--(TXD) PD1 		PC3 (ADC3)
   |  +-(INT0) PD2 		PC2 (ADC2)	TRIAC3
 ~-(----(INT1) PD3		PC1 (ADC1)	TRIAC2
   +------(T0) PD4		PC0 (ADC0)	TRIAC1
               VCC-+ +--AGND
               GND-(-+	AREF
             XTAL1 +----AVCC
             XTAL2		PB5 (SCK)   LED1 / Taster 1
          (T1) PD5		PB4 (MISO)  LED2 / Taster 2
        (AIN0) PD6		PB3 (MOSI)  LED3 / Taster 3
+-------(AIN1) PD7		PB2 (SS) 
+-[100k]-(ICP) PB0		PB1 (OC1)  
|
|   22F
+---||----+
+--[10k]--+---GND

             
			 RESET   V	PC5 (ADC5)
         (RXD) PD0  	PC4 (ADC4)
 	  +--(TXD) PD1 		PC3 (ADC3)
      +-(INT0) PD2 		PC2 (ADC2)
        (INT1) PD3		PC1 (ADC1)
Trffner (T0) PD4		PC0 (ADC0)
               VCC		AGND
               GND		AREF
             XTAL1		AVCC
             XTAL2		PB5 (SCK)   LED1
Klingel   (T1) PD5		PB4 (MISO)  LED2
Otreppe (AIN0) PD6		PB3 (MOSI)  LED3
ITreppe (AIN1) PD7		PB2 (SS)
         (ICP) PB0		PB1 (OC1)

**************************************************************************/

#include <avr/io.h>
#include <avr/interrupt.h>
#include <avr/pgmspace.h>
#include <avr/signal.h>
#include <avr/eeprom.h>
#include "main.h"
#include "protocol.h"
#include "hardware.h"


//#define TXHI sbi(DDRB,PB1);
//#define TXLO cbi(DDRB,PB1);



// #define ATMEGA8



unsigned char entprell[4];

unsigned char register tastdown asm("r8");
//unsigned char register dim_dir asm("r7"); // Richtung ob heller oder dunkler gedimmt werden soll
unsigned char dim_dir;
// Noisecanceler tastet den Komparatorausgang ab und filtert Spikes heraus



// Int1 als TX Control verwenden (fr Konvertermodul), ansonsten als Phasenanschnitt
// #define INT1_TXCTL





// wieviele Werte werden in der Filterkette betrachtet (nur 2er Potenzen)


// ******************* Hardware specific functions **************
void inline uart_send_byte(unsigned char b) {  // ca. 8 byte
		loop_until_bit_is_set(UCSRA, UDRE); // Prambel schicken		
		outp(b, UDR);
}


// Tasten abfragen, alle 1 mS eine andere, also alle 4 mS wird gesampled
void keyhandler(unsigned char i) { // ca 118 Byte
	unsigned char register eptmp;
	unsigned char register key;
	unsigned char register longkey;
	unsigned char register r;
	unsigned char register cfgtmp;
	
	r =  (PINB >> 3);  // Port mit den Tastern, 0 = gedrckt !!

	// Bitmasken berechnen
	longkey	=  (0x10 << i);  // longdown-Maske
	key 	=  longkey >> 4; //& 0x0F;	// down-Maske

	// in Temp-Variablen kopieren
	eptmp	=entprell[i];
	cfgtmp	= cfg[i];

	if (r & key) { // aktuelle Taste mit Maske abfragen
			// Taste i nicht gedrckt
			if (eptmp > 0) eptmp--;
	} else {
			// Taste i wurde gedrckt
			if (eptmp < 256-1) eptmp ++;  // hier schneller zhlen
			//eptmp &= 256-2;			        // 'runden' damit die Handler erreicht werden
	}


	if (eptmp == 8 && (tastdown & key)) { // Taste wurde los gelassen
		if (!(tastdown & longkey)) {
			// kurzer Druck
			cfgtmp  ^=  0x80;
		} 
		dim_dir ^=  key;				
		tastdown &= ~ key;
		tastdown &= ~ longkey;
		//talk(0x00,"EV",2);
		fifoin(255);
	}
	
	if (eptmp == 16 && !(tastdown & key)) { // Taste gedrckt, mind 32 mS
			tastdown |= key;
			

			fifoin(i); // Paket senden
	}
	

	if (eptmp > 250) { // Taste lang gedrckt, 500 mS

	
		//if (cfgtmp == 0xFF) dim_dir ^= key; // umdrehen
		//if (cfgtmp <= 0x80) dim_dir |= key; // aufwrts
		
		cfgtmp |= 0x80; // auf alle Flle einschalten
		if (dim_dir & key) {
			if (cfgtmp < 0xFF) cfgtmp++;
		} else {
			if (cfgtmp > 0x80) cfgtmp--;
		}

		tastdown |= longkey; // long keydown flag merken
		eptmp = 240;         // warten bis wieder lange gedrckt auf-ab intervall
	}




	// Register wieder in den Speicher zurck schreiben
	entprell[i]=eptmp;
	cfg[i] = cfgtmp;
}


int main(void) { // 96 byte

	init_hardware();
	//talk(0x00,"ON",2);
	fifoin(255);
	LED1OFF
	sei();

	for(;;) { // Hauptprogramm
		while(waitqueue) { // Diese Schleife wird alle 16 mS verlassen
			if (UCSRA & (1 << RXC))	{
				uart_receive_byte(inp(UDR));
			}
			//
			// hier weitere Programmteile einbauen. Kein Programmteil darf lnger als 8mS busy sein,
			// da sonst die Bytes nicht korrekt empfangen werden knnen
		};
		waitqueue=1;
		queuetimer();
	}
	return 0;

}

