#include "protocol.h"
#include "main.h"
#include "hardware.h"
#include "register.h"
#include <stdio.h>
// #include "debug.h"

#define MAX_RETRIES 8

#define ACK_FLAG 0x80

//#define WAIT usleep("10000");
#define WAIT asm("nop");

// Sendepuffer
unsigned char sendbuf[8];   // enthlt das aktuell zu sendende Paket
unsigned char sendlen=0;    // Lnge des zu sendeneden Pakets
unsigned char sendctl=0xFF; // 0xFF = Paket wurde erfolgreich verschickt, ansonsten Repeat-Counter
unsigned char senddelay=0;  // Delay bei Kollision
unsigned char sendsn;	    // Sequenznummer des Paketes

// Empfangspuffer
unsigned char rcvbuf[16]; // enthlt das empfangene Paket
unsigned char rcvlen;	
unsigned char rcvpacketnr;
unsigned char rcvsn;
unsigned char rcvcrc;
unsigned char rcvpos;


// nur 2er Potenzen
#define FIFOSIZE 8
unsigned char fifo[FIFOSIZE];
unsigned char readptr, writeptr;
unsigned char state;

#define PRIMZAHL 149
unsigned char random;

/*******************************************************
 * reiht ein Event in die Queue zum Versenden ein
 * sind mehr als FIFOSIZE Elemente in der Queue, so wird
 * das erste berschrieben
 *******************************************************/
void send_event(unsigned char i) {
	writeptr++;
	writeptr &= FIFOSIZE - 1;
	fifo[writeptr]=i;
}

/******************************************************
 * sofern kein weiteres Paet mehr zum Senden an steht, 
 * wird das nchste Paket vorbereitet
 ******************************************************/
 
void prepare_data(void) {
	if (sendctl < MAX_RETRIES) return; // es wird noch versucht ein Paket zu senden
	
	if(readptr != writeptr) { // es sind Pakete in der Queue
		readptr++;
		if (readptr == FIFOSIZE) readptr=0;
		
		sendbuf[0]=0x00; 	// Events zum Master senden
    	sendbuf[1]=DEV_ID;	// Absender sind wir
    	sendbuf[2]=sendsn++;
		sendbuf[3]='e';		// Paketcode: Event
		sendbuf[4]=fifo[readptr]; // Event. die obersten 4 Bit beschreiben den Eventtyp, die unteren eventuelle Parameter
		sendlen=5;		    // 5 Bytes Paketgre
    	senddelay=0;		// sofort senden
    	sendctl=0x00; 		// Senden starten
	}
}

/*************************************************************
 * sofern neue Daten in der Queue sind, werden diese versendet
 * weiterhin wird entsprechend lang gewartet, falls noch ein 
 * Paket zu versenden ist
 *************************************************************/
void sendqueue(void) { // wird periodisch alle 10 mS aufgerufen.
	prepare_data();

    if (senddelay > 0) {	// mssen wir warten
		senddelay--;
    } else {
		if (sendctl < MAX_RETRIES) { // ist das aktuelle Paket noch nicht versendet
	    	sendpacket(sendbuf, sendlen | (sendctl << 4));
	    	sendctl++; // Sendeversuch erhhen
	    	senddelay = (sendctl << 4) + (random & 0x0F); // 'zufllig' warten
			random += PRIMZAHL;
			if (sendctl == MAX_RETRIES) setup=fallback;
	  	}
    }
}

/**************************************************************
 * je nach buf die entsprechende Antwort generieren und Aktion
 * ausfhren
 * s <mem> <val> 			Setzt Speicherstelle <mem> auf <val>
 * t <chan>      			Toggled Channel 0..3
 * h <chan>					Schaltet Kanal 0..3 mit letzten Helligkeitswert ein
 * l <chan>					Schaltet Kanal 0..3 aus
 * d <chan> <hell> <ms>		Dimmt Kanal 0..3 auf Wert <hell> mit <ms> langen Schritten 
 *							-> Weitere Infos siehe Dimhandler
 * p <chan> <ms>			Kanal 0..3 ms lang auf High setzen
 * q <mem>					Speicherstelle abfragen
 * b <mem> <mem> <mem> <mem> Blockmode ersten 4 Speicherstellen
 * c 						show configuratin
 **************************************************************/
unsigned char inline do_action(unsigned char *buf, unsigned char *respond) {
	unsigned char respondlen=4; // default Antwortlnge
	unsigned char i;
	switch (buf[3] | 32 ) { // case insensitive

		case 's':	// set the current channel / memspace
			if (buf[4] < CFGSIZE) {
				cfg[ buf[4] ] = buf[5];
				respond[3]='!';
			}
		break;
		case 't': // toggle chan 0..4
			if (buf[4] < 4) {
				cfg[ buf[4] ] ^= 128;
				respond[3]='!';
			}
		break;
		case 'h': // chan 0..4 should be high
			if (buf[4] < 4) {
				cfg[ buf[4] ] |= 128;
				respond[3]='!';
			}
		break;
		case 'l': // chan 0..4 should be low
			if (buf[4] < 4) {
				cfg[ buf[4] ] |= 128;
				respond[3]='!';
			}
		break;
		case 'd': // chan 0..4 should be dimmed to <val> in <val> ms steps
			if (buf[4] < 4) {
				dim_to[ buf[4] ] = buf[5];
				dim_time[ buf[4] ] = buf[6];
				respond[3]='!';
			}
		break;
		case 'p': // chan 0..4 should be a Pulse of  <val>  * 100ms steps
			if (buf[4] < 4) {
				dim_to[ buf[4] ] = buf[5] & 0x7F;
				dim_time[ buf[4] ] = 100;
				respond[3]='!';
			}
		break;
		case 'q':	// query
			if (buf[4] < CFGSIZE) {
				respond[4]=cfg[buf[4]];
				respond[3]='=';
				respondlen=5;
			}
		break;
		case 'b': // block
			cfg[0] = buf[4];
			cfg[1] = buf[5];
			cfg[2] = buf[6];
			cfg[3] = buf[7];
			respond[3]='!';
		break;
		case 'c': // configuration
			respond[3]=':';
			for (i = 0; i < 12; i++) respond[i+4]=cfg[i];
			respondlen=12;
		break;
		case 'r': // raw
			switch (buf[4]) {
			 	case 0:
					OCR1A=buf[5];
					respond[3]='!';
					break;
				case 1:
					threshold=buf[5];
					respond[3]='!';
					break;
			}
		break;
	
	}
	return respondlen;
}

/*************************************************************************
 * ein gltiges Paket verarbeiten
 *************************************************************************/

void processrcvbuf(unsigned char *buf, unsigned char len, unsigned char packetnr) {
    unsigned int i;
	static unsigned char respond[16]; // Puffer um ein ACK-Paket zu verschicken
	static unsigned char respondlen;



    if (buf[0] == (DEV_ID | ACK_FLAG)) { // Paket mit Ack-Flag da

    	if (sendbuf[0] == buf[1] &&	// Passt das Ack-Paket zu dem das wir gerade gesendet haben
	  		sendbuf[2] == buf[2] ) {  // auch die Seqnr testen
		  	sendctl = 0xFF; 			// erfolgreich angekommen, keine weiteren Sendeversuche
		  	senddelay = 0;
      	}

    } else if (buf[0] == DEV_ID) { 	// kein Ack-Paket

		// unsere Antwort vorbereiten
		respond[0]=buf[1] | ACK_FLAG; // Neues Ziel = alte Quelle und MSB setzen, damit es als Ack-Paket erkannt wird
	  	respond[1]=DEV_ID;			  // unsere DEV_ID als Absender setzen
	  	respond[2] =buf[2];			  // Sequenznummer kopieren

	  	if (respondlen < 3) respondlen = 3; // Sanity check

	  	if ( packetnr == 0 || rcvsn != buf[2] ) { // neues (erstes) Paket empfangen, oder eins mit anderer SN
	    	rcvsn = buf[2];

	    	respond[3]='?'; // Default-Antwort
			respondlen=do_action(buf, respond);

	  	}
	  	for (i = 0; i < 100; i++) asm("nop");
	 	sendpacket(respond, respondlen); // Ack-Paket kann gleich verschickt werden, sollte das Ack-Paket
	  								  // verlorengehen, versucht die Gegenstelle das gleiche Paket nochmal zu senden

	  
    }
}

/*******************************************************
 * Statemachine um die Pakete zu erkennen
 *******************************************************/
void inline uart_receive_byte(unsigned char B) {

  switch (state) {
    case 0:	// erstes Syncbyte
	  if (B == 'R') {
	    state=2; 
		
	  } else {
	    state=0;
	  }
      break;
    
	case 1: // zweites Syncbyte
	  if (B == 'P') {
	    state=2; // Synced
	  } else {
	    state=0;
	  }
	break;
    
	case 2: // data len
	  rcvlen = B & 0x0F;
	  rcvpacketnr = (B & 0xF0) >> 4;
      state=3;
      rcvpos=0;
      rcvcrc=B;
    break;
    
	case 3: // Data processing
	  if (rcvpos == rcvlen) {
	    //state=4; // Data End 
		bytereceived=255;
	    if (rcvcrc == B) { // CRC OK

		  processrcvbuf(rcvbuf, rcvlen, rcvpacketnr);
	    } else {
		  // CRC-error
	    }
	    state=0;
	  } else {
	    rcvbuf[rcvpos++] = B;
	    rcvcrc ^= B;
	  }
	break;
    }
}

// Sendet ein Paket inkl Header, CRC direkt ber die UART
void sendpacket(unsigned char *buf, unsigned char len) {
    unsigned char crc;
    unsigned char pos;
	uart_rx_off();
	
    uart_send_byte('R');// Sync
    crc = len;
    uart_send_byte(len);
    for (pos = 0; pos  < (len & 0x0F); pos++) {
	uart_send_byte(buf[pos]);
        crc ^= buf[pos];
    }
    uart_send_byte(crc);
	while(!(UCSRA & (1 << UDRE))) keyhandler(); // wait until byte is sent
	uart_rx_on();
}
