#include "protocol.h"
#include "main.h"
#include "hardware.h"
#include <stdio.h>
#define MAX_RETRIES 8
#define DEV_ID 'B'


//#define WAIT usleep("10000");
#define WAIT asm("nop");

unsigned char sendbuf[8]; // enthlt das aktuell zu sendende Paket

unsigned char sendlen=0; // Lnge des zu sendeneden Pakets
unsigned char sendctl=0xFF; // 0xFF = Paket wurde erfolgreich verschickt, ansonsten Repeat-Counter
unsigned char senddelay=0; // Delay bei Kollision
unsigned char sendsn;	// Seriennummer des Paketes

unsigned char rcvbuf[16]; // enthlt das empfangene Paket
static unsigned char rcvlen;	
static unsigned char rcvpacketnr;
static unsigned char rcvsn;
static unsigned char rcvcrc;
static unsigned char rcvpos;

unsigned char respond[8];  // Antwort auf Befehle  
unsigned char respondlen;


// nur 2er Potenzen
#define FIFOSIZE 8
unsigned char fifo[FIFOSIZE];
unsigned char readptr, writeptr;
unsigned char state;


void fifoin(unsigned char i) {
	writeptr++;
	writeptr &= FIFOSIZE - 1;
	fifo[writeptr]=i;
}

void send_fifo(void) {
	if (sendctl < MAX_RETRIES) return; // Paket ist noch nicht besttigt
	if(readptr != writeptr) {
		readptr++;
		if (readptr == FIFOSIZE) readptr=0;
		
		sendbuf[0]=0x00; // Events zum Master senden
    	sendbuf[1]=DEV_ID;
    	sendbuf[2]=sendsn++;
		sendbuf[3]='e';
		sendbuf[4]=fifo[readptr];
		sendlen=5;
    	senddelay=0;
    	sendctl=0x00; // Senden starten
	}
}

void queuetimer(void) { // wird periodisch aufgerufen, 96 Byte
    if (senddelay > 0) {
		senddelay--;
    } else {
	if (sendctl < MAX_RETRIES) { // ist das aktuelle Paket noch nicht versendet
	    //printf("Sending try: %d\n", sendctl);
	    sendpacket(sendbuf, sendlen | (sendctl << 4));
	    sendctl++;
	    senddelay = (sendctl << 4) + (sendbuf[0] & 0x0F); // 'zufllig' warten
	  }
    }
}


// 328 Byte
void processrcvbuf(unsigned char *buf, unsigned char len) {
    unsigned int i;
    if (buf[0] == (DEV_ID | 0x80)) {
      if (sendbuf[0] == buf[1] &&
	  	sendbuf[2] == buf[2] ) {
		sendctl = 0xFF; // erfolgreich angekommen
		senddelay = 0;
	//printf("Ack-Paket empfangen\n");
      }
    } else {
	 if (buf[0] == DEV_ID) {
	  respond[0]=buf[1] | 0x80; // Neues Ziel = alte Quelle und MSB setzen, damit es als Ack-Paket erkannt wird
	  respond[1]=DEV_ID;		// unsere DEV_ID als Absender setzen
	  respond[2] =buf[2];			// Sequenznummer kopieren

	  if (respondlen < 3) respondlen = 3; // Sanity check
	  if ( rcvpacketnr == 0 || rcvsn != buf[2] ) { // neues Paket empfangen
	    rcvsn = buf[2];
	    respondlen=4;
	    respond[3]='?'; // Default-Antwort

		switch (buf[3] | 32 ) {
			case 's':
				if (buf[4] < CFGSIZE) {
					cfg[buf[4]] = buf[5];
					respond[3]='!';
				}
			break;
			case 'q':
				if (buf[4] < CFGSIZE) {
					respond[4]=cfg[buf[4]];
					respond[3]='=';
					respondlen=5;
				}
			break;
		}
	  }
	  for (i = 0; i < 1000; i++) asm("nop");
	 
	  sendpacket(respond, respondlen); // Ak-Paket kann gleich verschickt werden

	  
	 }
    }
}

// Receiver Function 164 Bytes
void inline uart_receive_byte(unsigned char B) {

  switch (state) {
    case 0:	// erstes Syncbyte
	if (B == 'R') {
	    state=1; 
		
	} else {
	    state=0;
	}
        break;
    case 1: // zweites Syncbyte
	if (B == 'P') {
	    state=2; 
	    //printf("\nSynced:");
		
	} else {
	    state=0;
	}
	break;
    case 2: // data len
	rcvlen = B & 0x0F;
	rcvpacketnr = (B & 0xF0) >> 4;
        state=3;
        rcvpos=0;
        rcvcrc=B;
        break;
    case 3: // Data processing
	if (rcvpos == rcvlen) {
	    state=4;
	    //printf("Received %d bytes: %s, ",rcvlen, rcvbuf);
	    if (rcvcrc == B) {
		//printf("CRC-OK \n");
		processrcvbuf(rcvbuf, rcvlen);
	    } else {
		//printf("CRC-Error\n");
	    }
	    state=0;
	} else {
	    rcvbuf[rcvpos++] = B;
	    rcvcrc ^= B;
	}
	break;
    }
}

// Sendet ein Paket inkl Header, CRC direkt ber die UART, 128 Byte
void sendpacket(unsigned char *buf, unsigned char len) {
    unsigned char crc;
    unsigned char pos;
    uart_send_byte('R');// Syncword
    uart_send_byte('P');
    crc = len;
    uart_send_byte(len);
    for (pos = 0; pos  < (len & 0x0F); pos++) {
	uart_send_byte(buf[pos]);
        crc ^= buf[pos];
    }
    uart_send_byte(crc);
}
