#include "protocol.h"
#include <stdio.h>
#define MAX_RETRIES 8
#define DEV_ID 0x00

unsigned char sendbuf[16]; // enthlt das aktuell zu sendende Paket

unsigned char sendlen=0; // Lnge des zu sendeneden Pakets
unsigned char sendctl=0xFF; // 0xFF = Paket wurde erfolgreich verschickt, ansonsten Repeat-Counter
unsigned char senddelay=0; // Delay bei Kollision
unsigned char sendsn;	// Seriennummer des Paketes

unsigned char rcvbuf[16]; // enthlt das empfangene Paket
static unsigned char rcvlen;	
static unsigned char rcvpacketnr;
static unsigned char rcvsn;
static unsigned char rcvcrc;
static unsigned char rcvpos;

unsigned char respond[8];  // Antwort auf Befehle
unsigned char respondlen;

unsigned char cfg[16];

unsigned char state;

unsigned char asciihex (unsigned char c) {
    if (c >= '0' && c <='9') return c-'0';
    if (c >= 'A' && c <='F') return c-'A'+10;
    if (c >= 'a' && c <='f') return c-'a'+10;
    return 0;
}

unsigned char hexascii (unsigned char c) {
    if (c >= 0 && c <= 9) return c+'0';
    if (c >= 0x0A && c <= 0x0F) return c+'A'-10;
    return 0;
}

void queuetimer(void) { // wird periodisch aufgerufen
    if (senddelay > 0) {
	senddelay--;
    } else {
	if (sendctl < MAX_RETRIES) { // ist das aktuelle Paket noch nicht versendet
	    printf("Sending try: %d\n", sendctl);
	    sendpacket(sendbuf, sendlen | (sendctl << 4));
	    sendctl++;
	    senddelay = sendctl * ((sendbuf[0] & 0x0F) + 16); // 'zufllig' warten
	}
    }
}

// talks to a module
unsigned char talk(unsigned char dest, unsigned char *buf, unsigned char len) {
    unsigned char sendbuf[16];
    unsigned char i;
    sendbuf[0]=dest;
    sendbuf[1]=DEV_ID;
    sendbuf[2]=sendsn++;
    for (i = 0; i < len; i++) sendbuf[i+3]=buf[i];
    return enqueuepacket(sendbuf, len + 3, 1);
}

unsigned char enqueuepacket(unsigned char *buf, unsigned char len, unsigned char block) {
    unsigned char i;
    for (i = 0; i < len; i++) sendbuf[i]=buf[i];
    sendlen=len;
    senddelay=0;
    if (block)
	while (sendctl < MAX_RETRIES) usleep(10000); // warte bis altes Paket gesendet oder nicht
    sendctl=0x00;
    printf("Packet enqueued %d\n", sendctl);
    if (block)
	while (sendctl < MAX_RETRIES) usleep(10000); // warte bis altes Paket gesendet oder nicht
    return sendctl;
}
unsigned char getanswer(unsigned char *buf) {
    unsigned char i;
    for (i = 0; i < rcvlen-3; i++) buf[i]=rcvbuf[i+3];
    return rcvlen-3;
}

void processrcvbuf(unsigned char *buf, unsigned char len) {
    unsigned char ackbuf[16];
    unsigned char i;
    if (buf[0] == (DEV_ID | 0x80)) {
      if (sendbuf[0] == buf[1] &&
	  sendbuf[2] == buf[2] ) {
	sendctl = 0xFF; // erfolgreich angekommen
	senddelay = 0;
	printf("Ack-Paket empfangen\n");
      }
    } else {
	if (buf[0] == DEV_ID) {
	  ackbuf[0]=buf[1] | 0x80;
	  ackbuf[1]=DEV_ID;
	  ackbuf[2]=buf[2];
	  printf("Rcvpnr%d\n",rcvpacketnr);
	  if ( rcvpacketnr == 0 || rcvsn != buf[2] ) {
	    rcvsn = buf[2];
	    printf("Gltiges Paket\n");
	    respondlen=1;
	    respond[0]='?';
	    switch(buf[3]) {
		case 'e':	// Ein Event ist eingegangen
		respond[0]='?';
		printf("---- Tastendruck %d 0x%02x\n", buf[1], buf[4]);
	    }
	  }
	  for (i = 0; i < respondlen; i++) ackbuf[i+3]=respond[i];
	  sendpacket(ackbuf,3+respondlen); // Ack-Paket kann gleich verschickt werden
	}
    }
}

// Receiver Function
void uart_receive_byte(unsigned char B) {
if (state < 3) printf("Rcv: %x",B);
  switch (state) {
    case 0:	// erstes Syncbyte
	if (B == 'R') {
	    state=1; 
	} else {
	    state=0;
	}
        break;
    case 1: // zweites Syncbyte
	if (B == 'P') {
	    state=2; 
	    printf("\nSynced:");
	} else {
	    state=0;
	}
	break;
    case 2: // data len
	rcvlen = B & 0x0F;
	rcvpacketnr = (B & 0xF0) >> 4;
        state=3;
        rcvpos=0;
        //if (B > 16) state=0;
        rcvcrc=B;
        break;
    case 3: // Data processing
	if (rcvpos == rcvlen) {
	    //rcvbuf[rcvlen]=0;
	    state=4;
	    //printf("Received %d bytes: %s, ",rcvlen, rcvbuf);
		printf("Dst: %c, Src %c, Len %d  %c%c%c%c%c CRC soll %d ist %d\n",rcvbuf[0], rcvbuf[1], rcvlen, rcvbuf[3],
		rcvbuf[4],rcvbuf[5],rcvbuf[6],rcvbuf[7], rcvcrc, B);
	    if (rcvcrc == B) {
		printf("CRC-OK \n");
		processrcvbuf(rcvbuf, rcvlen);
	    } else {
		printf("CRC-Error\n");
	    }
	    state=0;
	} else {
	    rcvbuf[rcvpos++] = B;
	    rcvcrc ^= B;
	}
	break;
    }
}

// Sendet ein Paket inkl Header, CRC direkt ber die UART
void sendpacket(unsigned char *buf, unsigned char len) {
    unsigned char crc;
    unsigned char pos;
    uart_send_byte('R');// Syncword
    uart_send_byte('P');
    crc=len;
    uart_send_byte(len);
    for (pos = 0; pos  < (len & 0x0F); pos++) {
	uart_send_byte(buf[pos]);
        crc ^= buf[pos];
    }
    uart_send_byte(crc);
}
